library window_with_time;

import 'dart:async';

class WindowWithTime<S, T> extends StreamEventTransformer {

  final Duration _duration;
  final Duration _interval;

  const WindowWithTime(this._duration, this._interval);

  Stream<Stream<T>> bind(Stream<S> stream) {
    var controller = new StreamController();
    var windows    = new Map();
    var counter    = 0;

    var slice = () {
      var id     = ++counter;
      var window = new StreamController();
      new Timer(this._duration, () {
        if (windows.containsKey(id)) {
          window.close();
          windows.remove(id);
        }
      });
      windows[id] = window;
      return window.stream;
    };
    controller.add(slice());

    var slicer = new Timer.periodic(this._interval, (_) {
      controller.add(slice());
    });

    stream.listen(
      (S data) {
        for (var window in windows.values) {
          window.add(data);
        }
      },
      onError: (error) {
        for (var window in windows.values) {
          window.addError(error);
        }
        controller.addError(error);
        slicer.cancel();
        windows.clear();
      },
      onDone: () {
        for (var window in windows.values) {
          window.close();
        }
        controller.close();
        slicer.cancel();
        window.clear();
      }
    );

    return controller.stream;
  }
}